@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013, 2014
+ * Copyright (c) 2013, 2014, 2019, 2020
  *      Inferno Nettverk A/S, Norway.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -44,7 +44,7 @@
 #include "common.h"
 
 static const char rcsid[] =
-"$Id: mother_util.c,v 1.22.4.8 2014/08/15 18:12:23 karls Exp $";
+"$Id: mother_util.c,v 1.22.4.8.6.3 2020/11/11 16:11:59 karls Exp $";
 
 /*
  * signal handler functions.  Upon reception of signal, "sig" is the real
@@ -110,10 +110,13 @@ mother_envsetup(argc, argv)
 
    for (fdreserved = 0;
    fdreserved < ELEMENTS(sockscf.state.reservedfdv);
-   ++fdreserved)
-      if ((sockscf.state.reservedfdv[fdreserved] = makedummyfd(0, 0)) == -1)
-         serr("%s: could not reserve fd #%lu for later use",
-              function, (unsigned long)fdreserved + 1);
+   ++fdreserved) {
+      if (sockscf.state.reservedfdv[fdreserved] == -1) {
+         if ((sockscf.state.reservedfdv[fdreserved] = makedummyfd(0, 0)) == -1)
+            serr("%s: could not reserve fd #%lu for later use",
+                 function, (unsigned long)fdreserved + 1);
+      }
+   }
 
    /*
     * close any descriptor we don't need, both in case of chroot(2)
@@ -1071,6 +1074,17 @@ sighup(sig, si, sc)
     */
    hostcacheinvalidate();
 
+#if HAVE_LDAP
+
+   /*
+    * LDAP cache entries depend on rule configuration so old cached entries
+    * might no longer be valid after a SIGHUP.  Need to invalidate cache
+    * at this point.
+    */
+   ldapcacheinvalid();
+
+#endif /* HAVE_LDAP */
+
    time_monotonic(&sockscf.stat.configload);
 
    slog(LOG_INFO,
